// ----------------------------------------------------------------------------------
//  Copyright 2003 by Digital Designs and Systems, Inc.  http://www.dideas.com
//  No warranty.  The user of this software assumes sole responsibly for the results
//  of execution.  This software may be freely distributed and modified so long as
//  this complete notice is included and unmodified.
// ----------------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include "args.h"
#include "tang274x.h"

struct s_options opt;



test_file(char *fname, int quiet) {
  FILE *fin;
  fin = fopen(fname, "r");
  
  if (fin) {
    fclose(fin);
    return 0;
  } else {
    if (!quiet) fprintf(stderr, "Can not open file %s\n", fname);
    return 1;
  }
}

args_help() {

    fprintf(stderr, "\nThere were errors processing the command line arugments.\n");
    fprintf(stderr, "Usage: tang274x.exe [-a<aperture>] [-g<gerber>] [-o<output>] [-D]\n");
    fprintf(stderr, "-a<aperture> sets the input aperture file, default %s\n",DEFAULT_APERTURE);
    fprintf(stderr, "-g<gerber> sets the input gerber file, default %s\n",DEFAULT_GERBER);
    fprintf(stderr, "-o<output> sets the RS274X output file, default %s\n",DEFAULT_OUTPUT);
    fprintf(stderr,"-D<level> set the debug level, default %s\n",DEFAULT_DEBUG);
    fprintf(stderr,"-q[qq] set the quiet level\n");
    fprintf(stderr,"-v print version and exit\n");
    fprintf(stderr,"-d<description> include layer description tag\n");
    fprintf(stderr,"-n include negative image command\n");
    fprintf(stderr,"\n");
}



int process_args(int narg, char **args) { 
  int i, err;
  char *s;

  err  = 0;



  opt.version = 0;
  opt.fname_aperture =  DEFAULT_APERTURE;
  opt.fname_gerber =   DEFAULT_GERBER;
  opt.fname_output =   DEFAULT_OUTPUT;
  opt.debug_level =  DEFAULT_DEBUG;
  opt.gerb_desc = "description";
  opt.quiet_level = 0;
  opt.polarity = POSITIVE_POLARITY;

  i = 1;
  while (i < narg) {
    s = args[i];
    if (*s == '-') {
      switch(s[1]) {
      case 'a' : // aperture file
	opt.fname_aperture = s+2;
	break;
      case 'g' : // gerber file
	opt.fname_gerber = s+2;
	break;
      case 'o' : // output file base
	opt.fname_output = s+2;
	break;
      case 'd' : // description
	opt.gerb_desc = s+2;
	break;
      case 'D' : // debug level
	opt.debug_level = atoi(s+2);
	break;
      case 'v' : // version and exit
	opt.version = 1;
	return 0;
	break;
      case 'n' : 
	opt.polarity = NEGATIVE_POLARITY;
	break;
      case 'q' : // quiet level
	opt.quiet_level = 1;
	if (s[2] == 'q') {
	  opt.quiet_level = 2;
	  if (s[3] == 'q') 	  opt.quiet_level = 3;
	}

	break;
      }
    } // if s==-
    else {
      fprintf(stderr,"Unknown option %s\n", s);
    }
    i++;
  } // while i < nargs

  //  printf("quiet level %d\n",opt.quiet_level);


  err += test_file(opt.fname_aperture, opt.quiet_level);
  err += test_file(opt.fname_gerber, opt.quiet_level);
  if (err) args_help();
  return err;
}

