// ----------------------------------------------------------------------------------
//  Copyright 2003 by Digital Designs and Systems, Inc.  http://www.dideas.com
//  No warranty.  The user of this software assumes sole responsibly for the results
//  of execution.  This software may be freely distributed and modified so long as
//  this complete notice is included and unmodified.
// ----------------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "tang274x.h"
#include "my_string.h"
#include "aprs.h"
#include "tango.h"
#include "rs274x.h"
#include "args.h"

int main(int narg, char **args) {
  int err;
  struct s_d_code_list *dcl;
  struct s_t_code_list *tcl;
  struct s_rs274x *fh_rs;

  err = process_args(narg, args);

  if (!opt.quiet_level || opt.version) printf("tang274x: Convert Tango gerber to RS274X : %s:%s:%s\n",VERSION, DATE_STR, TIME_STR);

  if (opt.version) {
    exit(0);
  }




  if (opt.debug_level) {
    printf("%s %s %s\n",opt.fname_aperture, opt.fname_gerber, opt.fname_output);
  }


  dcl = create_d_code_list(MAX_D_CODES);
  tcl = create_t_code_list(MAX_T_CODES);

  if (opt.debug_level) {
    printf("reading apertures.\n");
  }

  err = tango_read_apr(opt.fname_aperture, tcl, dcl);

  if (opt.debug_level) {
    display_tcodes(tcl);
    display_dcodes(dcl);
  }

  if (opt.quiet_level <= 1) printf("processing : %s [%d %d] + %s (%0.10s) -> %s\n", opt.fname_aperture, dcl->num_elems, tcl->num_elems, opt.fname_gerber, opt.gerb_desc, opt.fname_output);

  fh_rs = rs274x_create(opt.fname_output);
  rs274x_assign_tcode(fh_rs, tcl);
  rs274x_assign_dcode(fh_rs, dcl);
  rs274x_include_file(fh_rs, opt.fname_gerber, opt.gerb_desc, opt.polarity);
  rs274x_finished(fh_rs);




#if 0
  fh_rs = rs274x_create("c:/gnd.apr");
  rs274x_assign_tcode(fh_rs, tcl);
  rs274x_assign_dcode(fh_rs, dcl);
  rs274x_include_file(fh_rs, "q:/test/rpf3e.gnd","GND_SIDE_CU");
  rs274x_finished(fh_rs);

  fh_rs = rs274x_create("c:/top.apr");
  rs274x_assign_tcode(fh_rs, tcl);
  rs274x_assign_dcode(fh_rs, dcl);
  rs274x_include_file(fh_rs, "q:/dds2002b/cam/dds2002b.top","TOP_SIDE_CU");
  rs274x_finished(fh_rs);


  fh_rs = rs274x_create("c:/bot.apr");
  rs274x_assign_tcode(fh_rs, tcl);
  rs274x_assign_dcode(fh_rs, dcl);
  rs274x_include_file(fh_rs, "q:/dds2002b/cam/dds2002b.bot","BOT_SIDE_CU");
  rs274x_finished(fh_rs);

  fh_rs = rs274x_create("c:/top_silk.apr");
  rs274x_assign_tcode(fh_rs, tcl);
  rs274x_assign_dcode(fh_rs, dcl);
  rs274x_include_file(fh_rs, "q:/dds2002b/cam/dds2002b.tsl","TOP_SIDE_SILK");
  rs274x_finished(fh_rs);

  fh_rs = rs274x_create("c:/bot_silk.apr");
  rs274x_assign_tcode(fh_rs, tcl);
  rs274x_assign_dcode(fh_rs, dcl);
  rs274x_include_file(fh_rs, "q:/dds2002b/cam/dds2002b.bsl","BOT_SIDE_SILK");
  rs274x_finished(fh_rs);
#endif

  //  system("c:/bin/zp c:/test.zip c:/0rs274x.apr");



  return 0;
}
















