// ----------------------------------------------------------------------------------
//  Copyright 2003 by Digital Designs and Systems, Inc.  http://www.dideas.com
//  No warranty.  The user of this software assumes sole responsibly for the results
//  of execution.  This software may be freely distributed and modified so long as
//  this complete notice is included and unmodified.
// ----------------------------------------------------------------------------------


#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tang274x.h"
#include "aprs.h"
#include "rs274x.h"
#include "tango.h"


#define MAX_INCLUDE 20
struct s_rs274x *rs274x_create(char *fname) {
  struct s_rs274x *rs;
  FILE *fout;

  rs = (struct s_rs274x *)malloc(sizeof(struct s_rs274x));

  //  rs->fname = (char *)malloc(strlen(fname+1));
  //  strcpy(rs->fname, fname);
  rs->fname = fname;

  fout = fopen(fname,"w");
  if (!fout) {
    fprintf(stderr, "Can not open %s for writing\n", fname);
  }
  rs->fout = fout;
  rs->tcl = NULL;
  rs->dcl = NULL;

  rs->include_file = NULL;



  fprintf(fout,"G04 tang274x: Convert Tango gerber to RS274X : %s:%s:%s*\n",VERSION, DATE_STR, TIME_STR);

  fprintf(fout,"G04 created by dIDEAS - http://www.dideas.com (C) 2002*\n");
  fprintf(fout,"G04 *\n");

  fprintf(fout, "%%FSLAX23Y23*%%\n");
  fprintf(fout, "%%MOIN*%%\n");


#if 0
  fprintf(fout, "%%ASAXBY*\n");
  fprintf(fout, "MIA0B0*\n");
  fprintf(fout, "OFA0B0*\n");
  fprintf(fout, "SFA1.0B1.0*%%\n");
  fprintf(fout, "%%IJALBL*\n");
  fprintf(fout, "INX%s*\n",fname);
  fprintf(fout, "IOA0B0*\n");
  fprintf(fout, "IPPOS*\n");
  fprintf(fout, "IR0*%%\n");
#endif

  return rs;
}

int rs274x_assign_tcode(struct s_rs274x *rs, struct s_t_code_list *tcl) {
  rs->tcl = tcl;
  return 0;
}

int rs274x_assign_dcode(struct s_rs274x *rs, struct s_d_code_list *dcl) {
  rs->dcl = dcl;
  return 0;
}

int rs274x_write_dcodes(struct s_rs274x *rs) {
  int i, rcd;
  struct s_d_code *pdc;
  struct s_d_code_list *dcl;
  FILE *fout;

  fout = rs->fout;
  dcl = rs->dcl;
  //  printf("Write D_CODES %d %d\n", dcl->num_elems, dcl->max_elems);
  

  for(i=0; i < dcl->num_elems; i++) {
    pdc = dcl->dc+i;

    fprintf(fout,"G04 ADDING Aperture D%d X:%d Y:%d H:%d  T:%d*\n",pdc->code, pdc->x, pdc->y, pdc->hole, pdc->type);
    rcd = tango_aperture_to_rs274x_file(fout, pdc);

#if 0
      printf("Code:%d  Shape:%d  XY:%d %d   Hole:%d  Type:%d  Therm:OI:%d %d\n",
	     pdc->code, pdc->shape,
	     pdc->x, pdc->y,          
	     pdc->hole,       pdc->type, 
	     pdc->th_out,       pdc->th_in);
#endif
      
  }

  return 0;
}

int rs274x_include_file(struct s_rs274x *rs, char *fname, char *desc, int polarity) {
  FILE *fin;

  rs->include_file = fname;
  rs->desc = desc;
  rs->polarity = polarity;

  strcpy(rs->include_file, fname);
  //  printf("Added file :%s: to the include list\n", rs->include_file);


  fin = fopen(fname,"r");
  //    printf("Testing include file %s... ", fname);
  if (!fin) {
    printf("Can not open (%s).\n", fname);
  } else {
    //    printf("OK.\n");
  }
  fclose(fin);
  return 0;
}


#define BUFF_LEN 512
int file_append(FILE *fin, FILE *fout) {

  char buff[BUFF_LEN+1];
  char *p;
	   
  while(  (p = fgets(buff, BUFF_LEN, fin))  ) {
    fprintf(fout, "%s", p);
  }
  return 0;
}

int rs274x_process_include_file(struct s_rs274x *rs) {
  char *fname;
  FILE *fin, *fout;

  fout = rs->fout;
  fname = rs->include_file;

  // printf("processing %s\n", fname);

    fprintf(fout,"G04 *\n");
    fprintf(fout,"G04 *\n");
    fprintf(fout,"G04 INCLUDING FILE %s (%s)*\n",fname, rs->desc);

    if (rs->polarity == NEGATIVE_POLARITY) {
      fprintf(fout, "G04 : Negative Image*\n");
      fprintf(fout, "%%IPNEG*%%\n");
    }
    if (rs->polarity == POSITIVE_POLARITY) {
      fprintf(fout, "G04 : Positive Image*\n");
      fprintf(fout, "%%IPPOS*%%\n");
    }


  
  fin = fopen(fname,"r");
  if (!fin) {
    printf("Can not open include file %s\n", fname);
    fprintf(fout,"G04 ERROR - CAN NOT OPEN*\n");
    fprintf(fout,"G04 *\n");
    fprintf(fout,"G04 *\n");
  } else {
    fprintf(fout,"G04 *\n");
    fprintf(fout,"G04 *\n");

    //    fprintf(fout,"%%LN%s*%%\n", rs->desc);

    fprintf(fout,"%%IN%s*\n", rs->desc);
    fprintf(fout,"LPD*%%\n");
    file_append(fin, fout);
  }

  fclose(fin);
  return 0;
}



int rs274x_finished(struct s_rs274x *rs) {



  rs274x_write_dcodes(rs);
  rs274x_process_include_file(rs);


  fclose(rs->fout);
  free(rs);
  return 0;
}



