// ----------------------------------------------------------------------------------
//  Copyright 2003 by Digital Designs and Systems, Inc.  http://www.dideas.com
//  No warranty.  The user of this software assumes sole responsibly for the results
//  of execution.  This software may be freely distributed and modified so long as
//  this complete notice is included and unmodified.
// ----------------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "my_string.h"
#include "aprs.h"

#define LINE_IS_T_CODE     10
#define LINE_IS_D_CODE     11
#define LINE_IS_UNKNOWN    12



    // interesting lines have 4 spaces and either a Txx or Dxxx
// return a code related to the type of line
int tango_is_interesting(char *line) {
  int l,f;
  // for now just look for 4 spaces and a T or D
  l = strlen(line);
  if (l < 10) return LINE_IS_UNKNOWN;

  f = (line[0] == ' ') ? 1 : 0;
  f |= (line[1] == ' ') ? 2 : 0;
  f |= (line[2] == ' ') ? 4 : 0;
  f |= (line[3] == ' ') ? 8 : 0;
  if (f != 15) return LINE_IS_UNKNOWN;
  
  if (line[4] == 'T') { 
    //test for Tnn<space>
    if (isdigit(line[5]) && 
	isdigit(line[6]) && 
		line[7] == ' ') return LINE_IS_T_CODE;

    return LINE_IS_UNKNOWN;
  }

  if (line[4] == 'D') {
    // test for Dnn<space> or Dnnn<space>
    if (isdigit(line[5]) && 
	isdigit(line[6]) && 
		line[7] == ' ') return LINE_IS_D_CODE;

    if (isdigit(line[5]) && 
	isdigit(line[6]) && 
	isdigit(line[7]) && 
		line[8] == ' ') return LINE_IS_D_CODE;

    return LINE_IS_UNKNOWN;
  }

  return LINE_IS_UNKNOWN;
} // if_interesting



int tango_parse_t_code_line(char *line, struct s_t_code *tc) {
  char ch;
  int code, dia;
  //  code = atoi(line+5); 
  //  dia = atoi(line+8);
  sscanf(line," %c%d %d",&ch,&code,&dia);
  //  printf("%c %d %d\n",ch, code,dia);
  tc->code = code;
  tc->diameter = dia;
  return 0;
}

#define ROTATION 0
#define CROSS_HAIR_THICK 8 
#define CIR_LINE_THICK 8


int tango_aperture_to_rs274x_file(FILE *fout, struct s_d_code *pdc) {
  int rcd;

  char macro_name[30];
  rcd = 0;
  switch(pdc->shape) {
    case 1 : // ellipse
      if (pdc->x == pdc->y) {
	fprintf(fout, "%%ADD%0.3dC,0.%0.3dX0.%0.3d*%%\n",pdc->code, pdc->x, pdc->hole); 
      } else {
	fprintf(fout,"%%ADD%0.3dO,0.%0.3dX0.%0.3dX0.%0.3d*%%\n", pdc->code, pdc->x, pdc->y, pdc->hole);
      }
      break;
    case 2 : //drl sym

      // create a macro named "DRLSYM_D<DCODE>"
      sprintf(macro_name, "DRLSYM_D%0.3d", pdc->code);
      // Moire primitive
      // 1 x center
      // 2 y center
      // 3 outside diameter
      // 4 circle line thickness
      // 5 gap between circles
      // 6 number of circles
      // 7 cross hair thickness
      // 8 cross hair length
      // 9 rotation
      fprintf(fout, "%%AM%s*\n", macro_name);
      fprintf(fout,"6, 0, 0, 0.%0.3d, 0.%0.3d, 0, 1, 0.%0.3d, 0.%0.3d, 0.%0.3d*%%\n",pdc->x, 0, CROSS_HAIR_THICK, pdc->x, ROTATION);

      // now assign the macro to the D code
      fprintf(fout, "%%ADD%0.3d%s*%%\n",pdc->code, macro_name);


    break;
    case 3 : //thermal


      // thermal primitive 7 pg 29
      // 1 x center
      // 2 y  center
      // 3 outside dia
      // 4 inside dia
      // 5 cross hair thick
      // 6 rotation 

      // create a macro named "THERMAL_D<DCODE>"
      sprintf(macro_name, "THERMAL_D%0.3d", pdc->code);

      fprintf(fout, "%%AM%s*\n", macro_name);
      fprintf(fout,"7, 0.000, 0.000, 0.%0.3d, 0.%0.3d, 0.%0.3d, 0.%0.3d*%%\n", pdc->th_out, pdc->th_in,  CROSS_HAIR_THICK, ROTATION);

      // now assign the macro to the D code
      fprintf(fout, "%%ADD%0.3d%s*%%\n",pdc->code, macro_name);
      break;

    case 4 : // mthole

      // create a macro named "MTHOLE_D<DCODE>"
      sprintf(macro_name, "MTHOLE_D%0.3d", pdc->code);
      // Moire primitive
      // 1 x center
      // 2 y center
      // 3 outside diameter
      // 4 circle line thickness
      // 5 gap between circles
      // 6 number of circles
      // 7 cross hair thickness
      // 8 cross hair length
      // 9 rotation
      fprintf(fout, "%%AM%s*\n", macro_name);
      fprintf(fout,"6, 0, 0, 0.%0.3d, 0.%0.3d, 0, 1, 0.%0.3d, 0.%0.3d, 0.%0.3d*%%\n",pdc->x, CIR_LINE_THICK, CROSS_HAIR_THICK, pdc->x, ROTATION);

      // now assign the macro to the D code
      fprintf(fout, "%%ADD%0.3d%s*%%\n",pdc->code, macro_name);


    break;

  case 5 : // rdrect  


// round rectange macro:
// let x and y be the size of the "RD rectangle"
// assume x<y - if not then switch X and Y
// let 0,0 be the center of the aperture
// let r = X/4
// center a square of size X by Y-2r
// center a square of size 2r by Y
// place 4 circles of radius R, at +-R, +-(Y/2-R)
// DONE!
      // create a macro named "RNDRECT_D<DCODE>"
      sprintf(macro_name, "RNDRECT_D%0.3d", pdc->code);
      fprintf(fout, "%%AM%s*\n", macro_name);
      if (1) { 
	int x, y;
	x = pdc->x;
	y = pdc->y;
	if (x < y) {
	  int r,y2r,d;
	  r = x/4;
	  d = x/2;
	  y2r = y/2 -r;
	  fprintf(fout,"21, 1, 0.%0.3d,0.%0.3d, 0.000,0.000,0.000*\n", x, y-d);
	  fprintf(fout,"21, 1, 0.%0.3d,0.%0.3d, 0.000,0.000,0.000*\n", d, y);
	  fprintf(fout,"1, 1, 0.%0.3d, 0.%0.3d, 0.%0.3d*\n",d, r, y2r);
	  fprintf(fout,"1, 1, 0.%0.3d, 0.%0.3d, -0.%0.3d*\n",d, r, y2r);
	  fprintf(fout,"1, 1, 0.%0.3d, -0.%0.3d, -0.%0.3d*\n",d, r, y2r);
	  fprintf(fout,"1, 1, 0.%0.3d, -0.%0.3d, 0.%0.3d*%%\n",d, r, y2r);
	} else {
	  int r,x2r,d;
	  r = y/4;
	  d = y/2;
	  x2r = x/2 - r;
	  fprintf(fout,"21, 1, 0.%0.3d,0.%0.3d, 0.000,0.000,0.000*\n", x - 2*r, y);
	  fprintf(fout,"21, 1, 0.%0.3d,0.%0.3d, 0.000,0.000,0.000*\n", x, 2*r);
	  fprintf(fout,"1, 1, 0.%0.3d, 0.%0.3d, 0.%0.3d*\n",d, x2r, r);
	  fprintf(fout,"1, 1, 0.%0.3d, 0.%0.3d, -0.%0.3d*\n",d, x2r, r);
	  fprintf(fout,"1, 1, 0.%0.3d, -0.%0.3d, -0.%0.3d*\n",d, x2r, r);
	  fprintf(fout,"1, 1, 0.%0.3d, -0.%0.3d, 0.%0.3d*%%\n",d, x2r, r);
	}
      } // if 1
	
      // now assign the macro to the D code
      fprintf(fout, "%%ADD%0.3d%s*%%\n",pdc->code, macro_name);
    break;
    case 6 : // sq rect
      fprintf(fout,"%%ADD%0.3dR,0.%0.3dX0.%0.3dX0.%0.3d*%%\n", pdc->code, pdc->x, pdc->y, pdc->hole);
      break;

  case 7 : // target

      // create a macro named "TARGET_D<DCODE>"
      sprintf(macro_name, "TARGET_D%0.3d", pdc->code);
      // Moire primitive
      // 1 x center
      // 2 y center
      // 3 outside diameter
      // 4 circle line thickness
      // 5 gap between circles
      // 6 number of circles
      // 7 cross hair thickness
      // 8 cross hair length
      // 9 rotation
      if (1) {
	int gap, tmp, x, cir_outside_dia, n_cir;
	int ch_thick, thick;
	x = pdc->x;


	cir_outside_dia = (int) (x * 0.95) - 4;

	if (cir_outside_dia > (12 + 2*CIR_LINE_THICK)) {
	  n_cir = cir_outside_dia / (2*CIR_LINE_THICK);
	  if (n_cir > 4) n_cir = 4;
	  thick = CIR_LINE_THICK;
	  ch_thick = CROSS_HAIR_THICK;
	  gap = (cir_outside_dia - n_cir*thick) / n_cir / 2;
	} else {
	  if (cir_outside_dia < 4) cir_outside_dia = 0;
	  n_cir = 1;
	  thick = cir_outside_dia / 4;
	  if (thick < 1) thick = 1;
	  ch_thick = thick;
	  gap = (cir_outside_dia - thick) / 2;
	  if (gap < 5) {
	    n_cir= 0;
	    gap = 5;
	  }
	}

	fprintf(fout, "%%AM%s*\n", macro_name);
	fprintf(fout,"6, 0.000, 0.000, 0.%0.3d, 0.%0.3d, 0.%03d, %d, 0.%0.3d, 0.%0.3d, 0.%0.3d*%%\n", cir_outside_dia, thick, gap, n_cir, ch_thick, x, ROTATION);
      // now assign the macro to the D code
      fprintf(fout, "%%ADD%0.3d%s*%%\n",pdc->code, macro_name);
      } // if 1




    break;

  case 8 : // oval
      if (pdc->x == pdc->y) {
	// just use the circle
	fprintf(fout, "%%ADD%0.3dC,0.%0.3dX0.%0.3d*%%\n",pdc->code, pdc->x, pdc->hole); 
      } else {
	int x, y;

    // 2 cicles of diameter X
    // square thats X by Y-X
      // create a macro named "OVAL_D<DCODE>"
      sprintf(macro_name, "OVAL_D%0.3d", pdc->code);
      fprintf(fout, "%%AM%s*\n", macro_name);


	x = pdc->x;
	y = pdc->y;

	if (x < y) {
	  int r, dd, dd2;
	  r = x/2;
	  dd = y-x;
	  dd2 = dd/2;
	  fprintf(fout,"21, 1, 0.%0.3d,0.%0.3d, 0.000,0.000,0.000*\n", x, dd);
	  fprintf(fout,"1, 1, 0.%0.3d, 0.%0.3d, 0.%0.3d*\n",x, 0, dd2);
	  fprintf(fout,"1, 1, 0.%0.3d, 0.%0.3d, -0.%0.3d*%%\n",x, 0, dd2);
	} else {
	  int r, dd, dd2;
	  dd = x-y;
	  dd2 = dd/2;
	  r = y/2;
	  fprintf(fout,"21, 1, 0.%0.3d,0.%0.3d, 0.000,0.000,0.000*\n", dd, y);
	  fprintf(fout,"1, 1, 0.%0.3d, 0.%0.3d, 0.%0.3d*\n",y, dd2, 0);
	  fprintf(fout,"1, 1, 0.%0.3d, -0.%0.3d, 0.%0.3d*%%\n",y, dd2, 0);
	  
	}
	// now assign the macro to the D code
	fprintf(fout, "%%ADD%0.3d%s*%%\n",pdc->code, macro_name);
      } // if 1

    break;


  default : 
    fprintf(fout,"G04 Unknown aperture (Shape #%d) D%d X:%d Y:%d H:%d*\n",pdc->shape, pdc->code, pdc->x, pdc->y, pdc->hole);
    rcd = -1;
    break;
    } // swtich  
  return rcd;
}

int tango_parse_d_code_line(char *line, struct s_d_code *dc) {
  int code,x,y,hole;
  int n, sn, tn;
  int th_in, th_out;
  char ch;
  char shape[21], type[21], comment[52];

static char *shape_list[] = {
  "dummy", "ELLIPSE", "DRL SYM", "THERMAL", "MT HOLE", "RD RECT", "SQ RECT", "TARGET", "OVAL",  "XXXX" };
static char *type_list[] = { "dummy", "BOTH", "FLASH", "DRAW","XXXX" } ;

  n =   sscanf(line," %c%d %7c %d %d %d %19s %50c", &ch, &code, shape, &x, &y, &hole, type, comment);
  //  printf("%d %c %d %s %d %d %d %s :: %s :: ", n, ch, code, shape, x, y, hole, type, comment);

  // determine shape

  sn = list_match_nocase(shape_list, shape);
  //  printf("%d %s\n",sn,shape);
  tn = list_match_nocase(type_list, type);

  //  printf("%d %c %d %d %d %d %d %d ::: %s\n", n, ch, code, sn, x, y, hole, tn, comment);



  th_in = th_out = 0;
  // try read special code in the comment field for thermals.
  // skip the 7 chars where the shape is repeated then look for O_ and I_
  n =  sscanf(comment,"%*7c O_%d I_%d", &th_out, &th_in);
  //  if (n) printf("THERMAL : %d %d %d\n",n, th_in, th_out);


  // assign local vars to DC, thus returning them
  dc->code = code;
  dc->shape = sn;
  dc->x = x;
  dc->y = y;
  dc->hole = hole;
  dc->type = tn;
  dc->th_in = th_in;
  dc->th_out = th_out;

  return 0;
}

#define BUFF_LEN 511
int tango_read_apr(char *fname, struct s_t_code_list *tcl, struct s_d_code_list *dcl) {
  FILE *fin;
  char *p, buff[BUFF_LEN+1];
  int flg,err;
  int n_tcode, n_dcode;
  struct s_t_code *ptc;
  struct s_d_code *pdc;

  n_tcode = 0;
  n_dcode = 0;
  fin = fopen(fname,"r");
  if (!fin) {
    fprintf(stderr,"can not open file.\n");
    return -1;
  }

  while(  (p = fgets(buff, BUFF_LEN, fin))  ) {
    flg = tango_is_interesting(p);
    //    printf("%d %s",flg, p);

    switch(flg) {
    case LINE_IS_T_CODE :
      n_tcode = tcl->num_elems;
      if (n_tcode >= tcl->max_elems) {
	printf("T_CODE list is full [%d]\n",n_tcode);
	exit(-1);
      }
      tcl->num_elems++;      
      ptc = tcl->tc + n_tcode;
      err = tango_parse_t_code_line(p, ptc);
      //      printf("%d %d\n",ptc->code, ptc->diameter);
      break;


    case LINE_IS_D_CODE :
      n_dcode = dcl->num_elems;
      if (n_dcode >= dcl->max_elems) {
	printf("D_CODE list is full [%d]\n",n_dcode);
	exit(-1);
      }

      dcl->num_elems++;
      pdc = dcl->dc + n_dcode;
      err = tango_parse_d_code_line(p, pdc);

#if 0 // debugging code
      printf("Code:%d  Shape:%d  XY:%d %d   Hole:%d  Type:%d  Therm:OI:%d %d\n",
	     pdc->code, pdc->shape,
	     pdc->x, pdc->y,          
	     pdc->hole,       pdc->type, 
	     pdc->th_out,       pdc->th_in);
#endif

      // now add the d_code to the list
      
      break;
    default :
      break;
    } // switch flg


  } // while
    
  fclose(fin);

}
